"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useConnectForm = exports.handleConnectionFormFieldUpdate = void 0;
const react_1 = require("react");
const lodash_1 = require("lodash");
const mongodb_connection_string_url_1 = __importDefault(require("mongodb-connection-string-url"));
const validation_1 = require("../utils/validation");
const get_next_host_1 = require("../utils/get-next-host");
const default_connection_1 = require("../constants/default-connection");
const check_for_invalid_character_in_host_1 = require("../utils/check-for-invalid-character-in-host");
const connection_string_schema_1 = require("../utils/connection-string-schema");
const connection_ssh_handler_1 = require("../utils/connection-ssh-handler");
const tls_handler_1 = require("../utils/tls-handler");
const authentication_handler_1 = require("../utils/authentication-handler");
const connection_string_helpers_1 = require("../utils/connection-string-helpers");
function connectFormReducer(state, action) {
    switch (action.type) {
        case 'set-connection-form-state':
            return {
                ...state,
                ...action.newState,
            };
        case 'set-enable-editing-connection-string':
            return {
                ...state,
                enableEditingConnectionString: action.enableEditing,
            };
        case 'set-form-errors':
            return {
                ...state,
                errors: action.errors,
            };
    }
}
function parseConnectionString(connectionString) {
    const [parsedConnectionString, parsingError] = (0, connection_string_helpers_1.tryToParseConnectionString)(connectionString);
    return [
        parsedConnectionString,
        parsingError
            ? [
                {
                    fieldName: 'connectionString',
                    fieldTab: 'general',
                    message: parsingError.message,
                },
            ]
            : [],
    ];
}
function buildStateFromConnectionInfo(initialConnectionInfo) {
    const [, errors] = parseConnectionString(initialConnectionInfo.connectionOptions.connectionString);
    return {
        errors: errors,
        enableEditingConnectionString: initialConnectionInfo.connectionOptions.connectionString ===
            default_connection_1.defaultConnectionString && !initialConnectionInfo.lastUsed,
        warnings: errors
            ? []
            : (0, validation_1.validateConnectionOptionsWarnings)(initialConnectionInfo.connectionOptions),
        connectionOptions: (0, lodash_1.cloneDeep)(initialConnectionInfo.connectionOptions),
    };
}
function handleUpdateHost({ action, connectionStringUrl, connectionOptions, }) {
    const { newHostValue, fieldIndex } = action;
    try {
        (0, check_for_invalid_character_in_host_1.checkForInvalidCharacterInHost)(newHostValue, connectionStringUrl.isSRV);
        if (connectionStringUrl.hosts.length === 1 && newHostValue === '') {
            throw new Error('Host cannot be empty. The host is the address hostname, IP address, or UNIX domain socket where the mongodb instance is running.');
        }
        const updatedConnectionString = connectionStringUrl.clone();
        updatedConnectionString.hosts[fieldIndex] = newHostValue || '';
        const newConnectionStringUrl = new mongodb_connection_string_url_1.default(updatedConnectionString.toString());
        return {
            connectionOptions: {
                ...connectionOptions,
                connectionString: newConnectionStringUrl.toString(),
            },
            errors: [],
        };
    }
    catch (err) {
        return {
            connectionOptions: {
                ...connectionOptions,
                connectionString: connectionStringUrl.toString(),
            },
            errors: [
                {
                    fieldName: 'hosts',
                    fieldTab: 'general',
                    fieldIndex,
                    message: err.message,
                },
            ],
        };
    }
}
function handleConnectionFormFieldUpdate(action, currentConnectionOptions) {
    var _a;
    if (action.type === 'update-connection-string') {
        const [newParsedConnectionStringUrl, errors] = parseConnectionString(action.newConnectionStringValue);
        return {
            connectionOptions: {
                ...currentConnectionOptions,
                connectionString: (newParsedConnectionStringUrl === null || newParsedConnectionStringUrl === void 0 ? void 0 : newParsedConnectionStringUrl.toString()) ||
                    action.newConnectionStringValue,
            },
            errors,
        };
    }
    const [parsedConnectionStringUrl, errors] = parseConnectionString(currentConnectionOptions.connectionString);
    if (!parsedConnectionStringUrl) {
        return {
            connectionOptions: currentConnectionOptions,
            errors: errors,
        };
    }
    const updatedSearchParams = parsedConnectionStringUrl.typedSearchParams();
    switch (action.type) {
        case 'add-new-host': {
            const { fieldIndexToAddAfter } = action;
            const newHost = (0, get_next_host_1.getNextHost)(parsedConnectionStringUrl.hosts, fieldIndexToAddAfter);
            parsedConnectionStringUrl.hosts.splice(fieldIndexToAddAfter + 1, 0, newHost);
            if (updatedSearchParams.get('directConnection')) {
                updatedSearchParams.delete('directConnection');
            }
            return {
                connectionOptions: {
                    ...currentConnectionOptions,
                    connectionString: parsedConnectionStringUrl.toString(),
                },
            };
        }
        case 'remove-host': {
            const { fieldIndexToRemove } = action;
            parsedConnectionStringUrl.hosts.splice(fieldIndexToRemove, 1);
            if (parsedConnectionStringUrl.hosts.length === 1 &&
                !parsedConnectionStringUrl.hosts[0]) {
                parsedConnectionStringUrl.hosts[0] = `${default_connection_1.defaultHostname}:${default_connection_1.defaultPort}`;
            }
            return {
                connectionOptions: {
                    ...currentConnectionOptions,
                    connectionString: parsedConnectionStringUrl.toString(),
                },
            };
        }
        case 'update-tls': {
            return (0, tls_handler_1.handleUpdateTls)({
                action,
                connectionStringUrl: parsedConnectionStringUrl,
                connectionOptions: currentConnectionOptions,
            });
        }
        case 'update-tls-option': {
            return (0, tls_handler_1.handleUpdateTlsOption)({
                action,
                connectionStringUrl: parsedConnectionStringUrl,
                connectionOptions: currentConnectionOptions,
            });
        }
        case 'update-auth-mechanism': {
            return (0, authentication_handler_1.handleUpdateAuthMechanism)({
                action,
                connectionStringUrl: parsedConnectionStringUrl,
                connectionOptions: currentConnectionOptions,
            });
        }
        case 'update-username': {
            return (0, authentication_handler_1.handleUpdateUsername)({
                action,
                connectionStringUrl: parsedConnectionStringUrl,
                connectionOptions: currentConnectionOptions,
            });
        }
        case 'update-password': {
            return (0, authentication_handler_1.handleUpdatePassword)({
                action,
                connectionStringUrl: parsedConnectionStringUrl,
                connectionOptions: currentConnectionOptions,
            });
        }
        case 'update-host': {
            return handleUpdateHost({
                action,
                connectionStringUrl: parsedConnectionStringUrl,
                connectionOptions: currentConnectionOptions,
            });
        }
        case 'update-connection-schema': {
            const { isSrv } = action;
            try {
                const newConnectionStringUrl = (0, connection_string_schema_1.tryUpdateConnectionStringSchema)(parsedConnectionStringUrl, isSrv);
                return {
                    connectionOptions: {
                        ...currentConnectionOptions,
                        connectionString: newConnectionStringUrl.toString(),
                    },
                    errors: [],
                };
            }
            catch (err) {
                return {
                    connectionOptions: {
                        ...currentConnectionOptions,
                    },
                    errors: [
                        {
                            fieldName: 'isSrv',
                            fieldTab: 'general',
                            message: `Error updating connection schema: ${err.message}`,
                        },
                    ],
                };
            }
        }
        case 'update-ssh-options': {
            return (0, connection_ssh_handler_1.handleUpdateSshOptions)({
                action,
                connectionOptions: currentConnectionOptions,
            });
        }
        case 'update-search-param': {
            if (action.newKey) {
                const newValue = (_a = action.value) !== null && _a !== void 0 ? _a : updatedSearchParams.get(action.currentKey);
                updatedSearchParams.delete(action.currentKey);
                updatedSearchParams.set(action.newKey, newValue);
            }
            else {
                updatedSearchParams.set(action.currentKey, action.value);
            }
            return {
                connectionOptions: {
                    ...currentConnectionOptions,
                    connectionString: parsedConnectionStringUrl.toString(),
                },
            };
        }
        case 'update-auth-mechanism-property': {
            const authMechanismProperties = (0, connection_string_helpers_1.parseAuthMechanismProperties)(parsedConnectionStringUrl);
            if (action.value) {
                authMechanismProperties.set(action.key, action.value);
            }
            else {
                authMechanismProperties.delete(action.key);
            }
            const authMechanismPropertiesString = authMechanismProperties.toString();
            if (authMechanismPropertiesString) {
                updatedSearchParams.set('authMechanismProperties', authMechanismPropertiesString);
            }
            else {
                updatedSearchParams.delete('authMechanismProperties');
            }
            return {
                connectionOptions: {
                    ...currentConnectionOptions,
                    connectionString: parsedConnectionStringUrl.toString(),
                },
            };
        }
        case 'delete-search-param': {
            updatedSearchParams.delete(action.key);
            return {
                connectionOptions: {
                    ...currentConnectionOptions,
                    connectionString: parsedConnectionStringUrl.toString(),
                },
            };
        }
        case 'update-connection-path': {
            parsedConnectionStringUrl.pathname = action.value;
            return {
                connectionOptions: {
                    ...currentConnectionOptions,
                    connectionString: parsedConnectionStringUrl.toString(),
                },
            };
        }
        case 'remove-proxy-options': {
            const proxyOptions = [
                'proxyHost',
                'proxyPort',
                'proxyPassword',
                'proxyUsername',
            ];
            proxyOptions.forEach((key) => updatedSearchParams.delete(key));
            return {
                connectionOptions: {
                    ...currentConnectionOptions,
                    connectionString: parsedConnectionStringUrl.toString(),
                },
            };
        }
        case 'remove-ssh-options': {
            return {
                connectionOptions: {
                    ...currentConnectionOptions,
                    sshTunnel: undefined,
                },
            };
        }
    }
}
exports.handleConnectionFormFieldUpdate = handleConnectionFormFieldUpdate;
function useConnectForm(initialConnectionInfo, connectionErrorMessage) {
    const [state, dispatch] = (0, react_1.useReducer)(connectFormReducer, initialConnectionInfo, buildStateFromConnectionInfo);
    const setErrors = (0, react_1.useCallback)((errors) => {
        dispatch({
            type: 'set-form-errors',
            errors,
        });
    }, []);
    const setEnableEditingConnectionString = (0, react_1.useCallback)((enableEditing) => {
        dispatch({
            type: 'set-enable-editing-connection-string',
            enableEditing,
        });
    }, []);
    const updateConnectionFormField = (0, react_1.useCallback)((action) => {
        const updatedState = handleConnectionFormFieldUpdate(action, state.connectionOptions);
        dispatch({
            type: 'set-connection-form-state',
            newState: {
                ...state,
                errors: [],
                ...updatedState,
                warnings: updatedState.errors && updatedState.errors.length > 0
                    ? []
                    : (0, validation_1.validateConnectionOptionsWarnings)(updatedState.connectionOptions),
            },
        });
    }, [state]);
    setInitialState({
        connectionErrorMessage,
        initialConnectionInfo,
        setErrors,
        dispatch,
    });
    return [
        state,
        {
            updateConnectionFormField,
            setEnableEditingConnectionString,
            setErrors,
        },
    ];
}
exports.useConnectForm = useConnectForm;
function setInitialState({ connectionErrorMessage, initialConnectionInfo, setErrors, dispatch, }) {
    (0, react_1.useEffect)(() => {
        const { errors, enableEditingConnectionString, warnings, connectionOptions, } = buildStateFromConnectionInfo(initialConnectionInfo);
        dispatch({
            type: 'set-connection-form-state',
            newState: {
                errors,
                enableEditingConnectionString,
                warnings,
                connectionOptions,
            },
        });
    }, [initialConnectionInfo]);
    (0, react_1.useEffect)(() => {
        if (connectionErrorMessage) {
            setErrors([{ message: connectionErrorMessage }]);
        }
    }, [setErrors, connectionErrorMessage]);
}
//# sourceMappingURL=use-connect-form.js.map