"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateConnectionOptionsWarnings = exports.isSecure = exports.getConnectionString = exports.validateConnectionOptionsErrors = exports.errorMessageByFieldNameAndIndex = exports.fieldNameHasError = exports.errorMessageByFieldName = exports.errorsByFieldTab = void 0;
const mongodb_build_info_1 = require("mongodb-build-info");
const mongodb_connection_string_url_1 = __importDefault(require("mongodb-connection-string-url"));
function errorsByFieldTab(errors, tabId) {
    return errors.filter((error) => error.fieldTab === tabId);
}
exports.errorsByFieldTab = errorsByFieldTab;
function errorMessageByFieldName(errors, fieldName) {
    var _a;
    return (_a = (errors || []).find((err) => err.fieldName === fieldName)) === null || _a === void 0 ? void 0 : _a.message;
}
exports.errorMessageByFieldName = errorMessageByFieldName;
function fieldNameHasError(errors, fieldName) {
    return !!errorMessageByFieldName(errors, fieldName);
}
exports.fieldNameHasError = fieldNameHasError;
function errorMessageByFieldNameAndIndex(errors, fieldName, fieldIndex) {
    var _a;
    return (_a = (errors || []).find((err) => err.fieldName === fieldName && err.fieldIndex === fieldIndex)) === null || _a === void 0 ? void 0 : _a.message;
}
exports.errorMessageByFieldNameAndIndex = errorMessageByFieldNameAndIndex;
function validateConnectionOptionsErrors(connectionOptions) {
    const connectionString = getConnectionString(connectionOptions);
    return [
        ...validateAuthMechanismErrors(connectionString),
        ...(connectionOptions.sshTunnel
            ? validateSSHTunnelErrors(connectionOptions.sshTunnel)
            : validateSocksProxyErrors(connectionString)),
    ];
}
exports.validateConnectionOptionsErrors = validateConnectionOptionsErrors;
function validateAuthMechanismErrors(connectionString) {
    const authMechanism = connectionString
        .typedSearchParams()
        .get('authMechanism');
    switch ((authMechanism || '').toUpperCase()) {
        case '':
            return validateDefaultAuthMechanismErrors(connectionString);
        case 'MONGODB-X509':
            return validateX509Errors(connectionString);
        case 'GSSAPI':
            return validateKerberosErrors(connectionString);
        case 'PLAIN':
            return validateLDAPErrors(connectionString);
        case 'SCRAM-SHA-1':
        case 'SCRAM-SHA-256':
            return validateScramShaErrors(connectionString);
        default:
            return [];
    }
}
function validateScramShaErrors(connectionString) {
    return validateUsernameAndPasswordErrors(connectionString);
}
function validateDefaultAuthMechanismErrors(connectionString) {
    const { username, password } = connectionString;
    if (!username && !password) {
        return [];
    }
    return validateUsernameAndPasswordErrors(connectionString);
}
function validateUsernameAndPasswordErrors(connectionString) {
    const { username, password } = connectionString;
    const errors = [];
    if (!username) {
        errors.push({
            fieldTab: 'authentication',
            fieldName: 'username',
            message: 'Username is missing.',
        });
    }
    if (!password) {
        errors.push({
            fieldTab: 'authentication',
            fieldName: 'password',
            message: 'Password is missing.',
        });
    }
    return errors;
}
function validateX509Errors(connectionString) {
    const errors = [];
    if (!isSecure(connectionString)) {
        errors.push({
            fieldTab: 'tls',
            fieldName: 'tls',
            message: 'TLS must be enabled in order to use x509 authentication.',
        });
    }
    if (!connectionString.searchParams.has('tlsCertificateKeyFile')) {
        errors.push({
            fieldTab: 'tls',
            fieldName: 'tlsCertificateKeyFile',
            message: 'A Client Certificate is required with x509 authentication.',
        });
    }
    return errors;
}
function validateLDAPErrors(connectionString) {
    return validateUsernameAndPasswordErrors(connectionString);
}
function validateKerberosErrors(connectionString) {
    const errors = [];
    if (!connectionString.username) {
        errors.push({
            fieldTab: 'authentication',
            fieldName: 'kerberosPrincipal',
            message: 'Principal name is required with Kerberos.',
        });
    }
    return errors;
}
function validateSSHTunnelErrors(sshTunnel) {
    const errors = [];
    if (!sshTunnel.host) {
        errors.push({
            fieldTab: 'proxy',
            fieldName: 'sshHostname',
            message: 'A hostname is required to connect with an SSH tunnel.',
        });
    }
    if (!sshTunnel.password && !sshTunnel.identityKeyFile) {
        errors.push({
            fieldTab: 'proxy',
            fieldName: 'sshPassword',
            message: 'When connecting via SSH tunnel either password or identity file is required.',
        });
    }
    if (sshTunnel.identityKeyPassphrase && !sshTunnel.identityKeyFile) {
        errors.push({
            fieldTab: 'proxy',
            fieldName: 'sshIdentityKeyFile',
            message: 'File is required along with passphrase.',
        });
    }
    return errors;
}
function validateSocksProxyErrors(connectionString) {
    const searchParams = connectionString.typedSearchParams();
    const proxyHost = searchParams.get('proxyHost');
    const proxyPort = searchParams.get('proxyPort');
    const proxyUsername = searchParams.get('proxyUsername');
    const proxyPassword = searchParams.get('proxyPassword');
    const errors = [];
    if (!proxyHost && (proxyPort || proxyUsername || proxyPassword)) {
        errors.push({
            fieldTab: 'proxy',
            fieldName: 'proxyHostname',
            message: 'Proxy hostname is required.',
        });
        return errors;
    }
    return errors;
}
function getConnectionString(connectionOptions) {
    return new mongodb_connection_string_url_1.default(connectionOptions.connectionString);
}
exports.getConnectionString = getConnectionString;
function isSecure(connectionString) {
    const sslParam = connectionString.searchParams.get('ssl');
    const tlsParam = connectionString.searchParams.get('tls');
    if (!sslParam && !tlsParam) {
        return connectionString.isSRV;
    }
    return sslParam === 'true' || tlsParam === 'true';
}
exports.isSecure = isSecure;
function validateConnectionOptionsWarnings(connectionOptions) {
    const connectionString = getConnectionString(connectionOptions);
    return [
        ...validateReadPreferenceWarnings(connectionString),
        ...validateDeprecatedOptionsWarnings(connectionString),
        ...validateCertificateValidationWarnings(connectionString),
        ...validateDirectConnectionAndSrvWarnings(connectionString),
        ...validateDirectConnectionAndReplicaSetWarnings(connectionString),
        ...validateDirectConnectionAndMultiHostWarnings(connectionString),
        ...validateTLSAndHostWarnings(connectionString),
        ...validateSocksWarnings(connectionString),
    ];
}
exports.validateConnectionOptionsWarnings = validateConnectionOptionsWarnings;
function validateCertificateValidationWarnings(connectionString) {
    const warnings = [];
    if (isSecure(connectionString) &&
        (connectionString.searchParams.has('tlsInsecure') ||
            connectionString.searchParams.has('tlsAllowInvalidHostnames') ||
            connectionString.searchParams.has('tlsAllowInvalidCertificates'))) {
        warnings.push({
            message: 'Disabling certificate validation is not recommended as it may create a security vulnerability',
        });
    }
    return warnings;
}
function validateDeprecatedOptionsWarnings(connectionString) {
    const warnings = [];
    if (connectionString.searchParams.has('tlsCertificateFile')) {
        warnings.push({
            message: 'tlsCertificateFile is deprecated and will be removed in future versions of Compass, please embed the client key and certificate chain in a single .pem bundle and use tlsCertificateKeyFile instead.',
        });
    }
    return warnings;
}
function validateReadPreferenceWarnings(connectionString) {
    const warnings = [];
    const VALID_READ_PREFERENCES = [
        'primary',
        'primaryPreferred',
        'secondary',
        'secondaryPreferred',
        'nearest',
    ];
    const readPreference = connectionString.searchParams.get('readPreference');
    if (readPreference && !VALID_READ_PREFERENCES.includes(readPreference)) {
        warnings.push({
            message: `Unknown read preference ${readPreference}`,
        });
    }
    return warnings;
}
function validateDirectConnectionAndSrvWarnings(connectionString) {
    const warnings = [];
    if (connectionString.isSRV &&
        connectionString.searchParams.get('directConnection') === 'true') {
        warnings.push({
            message: 'directConnection not supported with SRV URI.',
        });
    }
    return warnings;
}
function validateDirectConnectionAndReplicaSetWarnings(connectionString) {
    const warnings = [];
    if (connectionString.searchParams.get('replicaSet') &&
        connectionString.searchParams.get('directConnection') === 'true') {
        warnings.push({
            message: 'directConnection is not supported with replicaSet.',
        });
    }
    return warnings;
}
function validateDirectConnectionAndMultiHostWarnings(connectionString) {
    const warnings = [];
    if (connectionString.hosts.length > 1 &&
        connectionString.searchParams.get('directConnection') === 'true') {
        warnings.push({
            message: 'directConnection is not supported with multiple hosts.',
        });
    }
    return warnings;
}
function validateTLSAndHostWarnings(connectionString) {
    const warnings = [];
    const nonLocalhostsCount = connectionString.hosts.filter((host) => !(0, mongodb_build_info_1.isLocalhost)(host)).length;
    if (nonLocalhostsCount && !isSecure(connectionString)) {
        warnings.push({
            message: 'Connecting without tls is not recommended as it may create a security vulnerability.',
        });
    }
    return warnings;
}
function validateSocksWarnings(connectionString) {
    const warnings = [];
    const searchParams = connectionString.typedSearchParams();
    const proxyHost = searchParams.get('proxyHost');
    if (!proxyHost || (0, mongodb_build_info_1.isLocalhost)(proxyHost)) {
        return warnings;
    }
    if (searchParams.get('proxyPassword')) {
        warnings.push({
            message: 'Socks5 proxy password will be transmitted in plaintext.',
        });
    }
    if (connectionString.hosts.find(mongodb_build_info_1.isLocalhost)) {
        warnings.push({
            message: 'Using remote proxy with local MongoDB service host.',
        });
    }
    return warnings;
}
//# sourceMappingURL=validation.js.map